#![allow(non_snake_case)]

use dioxus::events::*;
use dioxus::prelude::*;

const STYLE: &str = include_str!("app.css");

fn main() {
    dioxus::desktop::launch(App);
}

fn App(cx: Scope) -> Element {
    cx.render(rsx!(
        style { "{STYLE}" }
        h1 { "Chrust" }
        ChatPane {}
    ))
}

#[inline_props]
fn MessageLog<'a>(cx: Scope, messages: &'a [String]) -> Element {
    let rendered_messages = messages
        .iter()
        .map(|message| rsx!(p { "align": "right", "{message}" }));
    cx.render(rsx! {
        div {
            rendered_messages
        }
    })
}

#[derive(Props)]
struct MessageComposerProps<'a, F: 'a> {
    onsend: &'a F,
}

fn MessageComposer<'a, F: Fn(String)>(cx: Scope<'a, MessageComposerProps<'a, F>>) -> Element<'a> {
    let message = use_state(&cx, String::new);
    let send_message = move || {
        (cx.props.onsend)(message.get().clone());
        dbg!(message);
        message.set(String::new());
    };

    cx.render(rsx! {
        div {
            class: "message-composer",
            width: "100%",
            input { class: "chat-message-input",
                value: "{message}",
                oninput: move |evt| message.set(evt.value.clone()),
                onkeydown: move |evt| if let KeyCode::Enter = evt.key_code { send_message() },
                }
            button { class: "chat-message-send-button",
                onclick: move |_| send_message(),
                "Send"
                }
        }
    })
}

fn ChatPane(cx: Scope) -> Element {
    let previous_messages = use_state(&cx, Vec::new);

    cx.render(rsx! {
        MessageLog { messages: previous_messages.get().as_slice() }
        MessageComposer { onsend: move |s| previous_messages.modify().push(s) }
    })
}
